package main

import (
    "flag"
    "log"
    "fmt"
    "net/http"
    "os"
    "strings"
    "sync"
    "time"
)

const usage = `Usage: https_server

Serve a sample page over HTTPs.

positional arguments
  [HOST]:PORT

  A "host:port" address taht the target listson on for HTTPS requests.

options:
  -cert TLS_CERT
    The server's TLS certificate file.

    Default: server.crt

  -key TLS_PRIVATE_KEY
    The server's TLS private key.
   
    Default: server.key

  -help
    Display this usage statement and exit

example:
$ ./https_server -cert my.cert -key my.key 127.0.0.1:12345
`

type Options struct {
    certFile string
    keyFile string
    hostport string
}

func die(format string, a ...any) {
    if !strings.HasSuffix(format, "\n") {
        format += "\n"
    }
    fmt.Fprintf(os.Stderr, format, a...)
    os.Exit(1)
}

func printUsage() {
    fmt.Fprintf(os.Stderr, "%s", usage)
}

func parseOptions() *Options {
    options := Options{}

    flag.Usage = printUsage
    flag.StringVar(&options.certFile, "cert", "server.crt", "")
    flag.StringVar(&options.keyFile, "key", "server.key", "")

    flag.Parse()

    if flag.NArg() != 1 {
        die("expected one positional argument but got %d", flag.NArg())
    }

    options.hostport = flag.Arg(0)

    return &options
}


type VisitHandler struct {
    mtx sync.Mutex // guards numVisits
    numVisits int
}

func (h *VisitHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
    h.mtx.Lock()
    defer h.mtx.Unlock()
    h.numVisits += 1

    w.Header().Set("Content-Type", "text/plain")
    w.WriteHeader(http.StatusOK)
    fmt.Fprintf(w, "You are visitor #%d\n", h.numVisits)
}

func newServer(hostport string) *http.Server {
    mux := http.NewServeMux()
    h := &VisitHandler{}
    mux.Handle("/visit", h)

    s := &http.Server {
        Addr: hostport,
        ReadTimeout: 30 * time.Second,
        WriteTimeout: 30 * time.Second,
        IdleTimeout: 30 * time.Second,
        Handler: mux,
    }

    return s
}

func main() {
    options := parseOptions()

    s := newServer(options.hostport)

    log.Printf("About to listen on %s.  Go to https://%s/",
            options.hostport, options.hostport)

    err := s.ListenAndServeTLS(options.certFile, options.keyFile)
    log.Fatal(err)
}
